/* dia-canvas-element.h
 * Copyright (C) 2001  Arjan Molenaar
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
/*
 * DiaCanvasElement
 * ----------
 * Base class for element like objects, which includes basically everything
 * that's not a line.
 * Elements have eight handles around them and can move, but handles can not
 * connect to other elements. Moving an individual handle will cause the
 * element to call DiaCanvasElementClass::resize().
 */
 /*Added some new features, like where the object can be connected*/

#ifndef __DIA_CANVAS_ELEMENT_H__
#define __DIA_CANVAS_ELEMENT_H__

#include <diacanvas/dia-canvas.h>

G_BEGIN_DECLS

typedef enum {
	DIA_HANDLE_N,
	DIA_HANDLE_NW,
	DIA_HANDLE_NE,
	DIA_HANDLE_S,
	DIA_HANDLE_W,
	DIA_HANDLE_E,
	DIA_HANDLE_SW,
	DIA_HANDLE_SE
} DiaCanvasElementHandle;

/* enum {
	DIA_NEED_ALIGN_HANDLES = 1 << 5
}; */

#define DIA_TYPE_CANVAS_ELEMENT        (dia_canvas_element_get_type ())
#define DIA_CANVAS_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), DIA_TYPE_CANVAS_ELEMENT, DiaCanvasElement))
#define DIA_CANVAS_ELEMENT_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), DIA_TYPE_CANVAS_ELEMENT, DiaCanvasElementClass))
#define DIA_IS_CANVAS_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIA_TYPE_CANVAS_ELEMENT))
#define DIA_IS_CANVAS_ELEMENT_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), DIA_TYPE_CANVAS_ELEMENT))
#define DIA_CANVAS_ELEMENT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), DIA_TYPE_CANVAS_ELEMENT, DiaCanvasElementClass))

typedef struct _DiaCanvasElement DiaCanvasElement;
typedef struct _DiaCanvasElementClass DiaCanvasElementClass;

struct _DiaCanvasElement
{
	DiaCanvasItem item;
	
	/* These two values describe the size of the element. They also
	 * tell us where the handles should be placed. */
	gdouble width, height;
	gdouble min_width, min_height;
       /*change size¿?*/
        int resizable;


};

struct _DiaCanvasElementClass
{
	DiaCanvasItemClass parent_class;

/*
 * relative_angle_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el ángulo relativo, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor entero con el ángulo 0-360 grados
 *
*/
int (*relative_angle_from)(double x, double y);

/*
 * relative_hypotenusa_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el valor de la hipotenusa, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor double 
 *
*/
double (*relative_hypotenusa_from)(double x, double y);

/*
 * relocalize_point_from:
 * @DiaCanvasItem: *item
 * @gdouble *x
 * @gdouble *y
 * Recibe  un par de puntos en coordenadas absolutas
 *   y los cambia de acuerdo a las coordenadas del ítem
 * traslación del origen del plano de coordenadas
*/
void (*relocalize_point_from)(DiaCanvasItem *item,gdouble *x,gdouble *y);

  }; /*class end :) */


/*delette a conexion point from the obj
 * if the angle that contains that conexión poitns does no is exist
 * do not nothing
 *Elimina un águlo de conexión y su tolerancia
 * esta función debería llamarse 
 * para cuando se desea retirar un  ángulo
 * de conexión de un ítem determinado
 * el ángulo ddebe ser el relativo. 
 * delete a  angle conexion point
 * must be used if wants to retires a 
 * conexion point from a Item
*/

void dia_canvas_element_del_cnx_p(DiaCanvasItem *item, int angle);


/* 
 *Pregunta si en el ángulo pasado como parámetro
 *tiene permiso de conexión.
 * entrega 0 para No y Uno para sí.
 * Incluye el 
 *También entrega 1 cuando no tiene ángulos seleccionados
 * A su vez le pasa el handle del ítem que se le va a conectar 
 * y le pregunta  si está en la cabeza o cóla
 */
int dia_canvas_element_exits_cnx_point_h(DiaCanvasItem *item, int angle,DiaHandle *handle_sagitta);


/*
 * entrega el ángulo oficial de conexión 
 * sí se le pasa como parámetro un 
 * ángulo que se encuentra en el dominio
 *Se le pasa el íem(cuadro) y el ángulo de pregunta
*/

int dia_canvas_element_angle_off_segment(DiaCanvasItem *item,int angle_p);


/*guarda un águlo de conexión, su tolerancia y su flujo
 *esta función debería llamarse 
 * para cuando se desea poner un nuevo ángulo
 * relativo.
 * de conexión para un ítem determinado
 * inserts a new angle conexion point
 * must be used if wants to puts a 
 * conexion point from a Item
 * OJO, CUANDO SE REDUZCA EL OBJETO, NO PUEDE SER CERO
*/
int dia_canvas_element_insert_new_cnx_p(DiaCanvasItem *item, int angle, int tol, int fluxus);



/*Función que entrega el ángulo en el que se encuentra
 * con respecto al ítem
 * de una línea y un cuadro
 * pero se le pasa el manejador de la línea
*/

int dia_canvas_element_line_angle(DiaCanvasItem *item,DiaHandle *handle_sagitta);

GType	dia_canvas_element_get_type (void);

DiaHandle* dia_canvas_element_get_opposite_handle (DiaCanvasItem *item,
						   DiaHandle *handle);


/*< Protected >*/
void	dia_canvas_element_align_handles (DiaCanvasElement *element);

G_END_DECLS


#endif /* __DIA_CANVAS_ELEMENT_H__ */
